/*
   This ast2infix() method takes an Abstract Syntax Tree
   for Language-14t and converts the tree into a string that
   contains a "C like" syntax, including infix notation for
   the arithmetic and boolean expressions.

   ast2infix() uses knowledge about operator precedence (in infix
   notation) to use the minimun number of parentheses.
*/

public class AST2infix
{
   public static String ast2infix(Tree tree)
   {
      return ast2infix( tree, 0, true );
   }


   private static String ast2infix(Tree tree, int indentation, boolean inline)
   {
      String indent = "";
      for (int i = 0; i < indentation; i++)
         indent += " ";

      String node = tree.getElement();
      String result = "";


      if ( node.equals("prog") )
      {
         result += "Program\n{\n";
         indentation += 3;

         // process each definition
         for (int i = 0; i < tree.degree(); i++)
         {
            result += ast2infix( tree.getSubTree(i), indentation, false );
            result += "\n";
         }

         result += indent + "}";
      }
      else if ( node.equals("var") )
      {
         Tree type = tree.getSubTree(0);

         String name = tree.getSubTree(1).getElement();

         if ( type.getElement().equals("signature") )  // function definition
         {
            // output function type and function name
            result += ast2infix( type, indentation, true );
            result += "\n";
            result += indent;
            result += name;
            // now output the function definition
            if( tree.getSubTree(2).getElement().equals("lambda") )
            {  // function is defined using a lambda expression
               result += "(";
               // get the formal parameter list
               if ( tree.getSubTree(2).degree() > 1 )
               {
                  result += tree.getSubTree(2).getSubTree(1).getElement();
                  for (int i = 2; i < tree.getSubTree(2).degree()-1; i++)
                  {
                     result += ", " + tree.getSubTree(2).getSubTree(i).getElement();
                  }
               }
               result += ")\n";

               // get the body of the function
               Tree expr = tree.getSubTree(2).getSubTree(tree.getSubTree(2).degree()-1);
               if ( ! expr.getElement().equals("begin") )
               {
                  result += indent + "{\n";
                  indentation += 3;
               }
               result += ast2infix( expr, indentation, false );
               if ( ! expr.getElement().equals("begin") )
               {
                  result += ";\n" + indent + "}";
               }
               result += "\n";
            }
            else  // function is not defined using a lambda expression
            {
               result += " = ";
               result += ast2infix( tree.getSubTree(2), indentation, true );
               result += "\n";
            }
         }
         else // variable definition
         {
            indentation += 7 + name.length();
            result += "var " + name + " : " + type.getElement() + " = "
                             + ast2infix( tree.getSubTree(2), indentation, true );
         }
      }
      else if ( node.equals("signature") )
      {
         String rtnType = tree.getSubTree(0).getElement();
         if ( rtnType.equals("signature") )
         {
            rtnType = ast2infix( tree.getSubTree(0), 0, true );
         }
         result += "(";
         for (int i = 1; i < tree.degree(); i++)
         {
            if ( tree.getSubTree(i).getElement().equals("signature") )
            {
               result += ast2infix( tree.getSubTree(i), indentation, true );
            }
            else
            {
               result += tree.getSubTree(i).getElement();
            }
            if ( i < tree.degree()-1 )
            {
               result += ", ";
            }
         }
         result += ")";
         result += " => " + rtnType;
      }
      else if ( node.equals("lambda") )  // used for anonymous function application only
      {
         // get the list of parameter types
         Tree types = tree.getSubTree(0);

         // get the formal parameter list
         result += "(";
         for (int i = 1; i < tree.degree()-1; i++)
         {
            result += tree.getSubTree(i).getElement()
                       + " : " + types.getSubTree(i).getElement();
            if ( i < tree.degree()-2 )
            {
               result += ", ";
            }
         }

         result += ") : " + types.getSubTree(0).getElement();

         // get the body of the function
         result += " -> ";
         Tree expr = tree.getSubTree(tree.degree()-1);
         result += ast2infix( expr, indentation, true );
      }
      else if ( node.equals("apply") )
      {
         String name = tree.getSubTree(0).getElement();
         if ( name.equals("+")
           || name.equals("-")
           || name.equals("neg")
           || name.equals("*")
           || name.equals("/")
           || name.equals("%")
           || name.equals("^")
           || name.equals("==")
           || name.equals("!=")
           || name.equals("<=")
           || name.equals(">=")
           || name.equals("<")
           || name.equals(">")
           || name.equals("&&")
           || name.equals("||")
           || name.equals("!") )
         {
            result += builtin( tree, indentation, inline );
         }
         else
         {
            if ( tree.getSubTree(0).getElement().equals("lambda") )
            { // the anonymous lambda expression case
               result += "("+ast2infix( tree.getSubTree(0), indentation, true )+")";
            }
            else // the named function case
            {
               result += ast2infix( tree.getSubTree(0), indentation, true );
            }

            indentation += 3;
            // get the actual parameter list
            result += "(";
            if ( tree.degree() > 1 )
            {
               result += ast2infix( tree.getSubTree(1), indentation, true );
               for (int i = 2; i < tree.degree(); i++)
               {
                  result += ", " + ast2infix( tree.getSubTree(i), indentation, true );
               }
            }
            result += ")";
         }
      }
      else if ( node.equals("print") )
      {
         indentation += 8;
         result += "print(" + ast2infix( tree.getSubTree(0), indentation, true ) + ")";
      }
      else if ( node.equals("if") )
      {
         result += "if (" + ast2infix( tree.getSubTree(0), indentation+7, true ) + ")\n";
         if ( tree.getSubTree(1).getElement().equals("begin") )
         {
            result += ast2infix( tree.getSubTree(1), indentation, false ) + "\n";
         }
         else
         {
            result += ast2infix( tree.getSubTree(1), indentation+3, false ) + "\n";
         }
         result += indent + "else\n";
         if ( tree.getSubTree(1).getElement().equals("begin") )
         {
            result += ast2infix( tree.getSubTree(2), indentation, false );
         }
         else
         {
            result += ast2infix( tree.getSubTree(2), indentation+3, false );
         }
      }
      else if ( node.equals("while") )
      {
         result += "while (" + ast2infix( tree.getSubTree(0), indentation+7, true ) + ")\n";

         if ( tree.getSubTree(1).getElement().equals("begin") )
         {
            result += ast2infix( tree.getSubTree(1), indentation, false );
         }
         else
         {
            result += ast2infix( tree.getSubTree(1), indentation+3, false );
         }
      }
      else if ( node.equals("begin") )
      {
         result += "{\n";
         indentation += 3;

         // process each expression
         for(int i = 0; i < tree.degree(); i++)
         {
            result += ast2infix( tree.getSubTree(i), indentation, false );
            if ( tree.getSubTree(i).getElement().equals("begin") )
               result += "\n";
            else
               result += ";\n";
         }

         result += indent + "}";
      }
      else if ( node.equals("set") )
      {
         indentation += 3 + tree.getSubTree(0).getElement().length();
         result += tree.getSubTree(0).getElement()
                   + " = " + ast2infix( tree.getSubTree(1), indentation, true );
      }
      else if ( tree.degree() == 0 )  // constant term
      {
         result += node;
      }

      if (!inline)
         result = indent + result;

      return result;
   }//ast2infix()


   private static String builtin(Tree tree, int indentation, boolean inline)
   {
      String result = "";

      String op = tree.getSubTree(0).getElement();

      if ( tree.degree() == 2 )  // unary operator
      {
         if ( op.equals("neg") ) { op = "-"; }

         if ( tree.getSubTree(1).degree() >= 2 )
         {
            result += op + "(" + ast2infix( tree.getSubTree(1), indentation, true ) + ")";
         }
         else
         {
            result += op + ast2infix( tree.getSubTree(1), indentation, true );
         }
      }
      else if ( tree.degree() == 3 )  // binary operator
      {
         int middlePrecedence = precedence( tree );
         int leftPrecedence = precedence( tree.getSubTree(1) );

         if ( leftPrecedence > middlePrecedence )
         {
            result += "(" + ast2infix( tree.getSubTree(1), indentation, true ) + ")";
         }
         else
         {
            result += ast2infix( tree.getSubTree(1), indentation, true );
         }

         result += " " + op + " ";

         int rightPrecedence = precedence( tree.getSubTree(2) );

         if ( rightPrecedence >= middlePrecedence )
         {
            result += "(" + ast2infix( tree.getSubTree(2), indentation, true ) + ")";
         }
         else
         {
            result += ast2infix( tree.getSubTree(2), indentation, true );
         }
      }
      else  // operators with 3 or more operands
      {
         int operatorPrecedence = precedence( tree );
         int leftPrecedence = precedence( tree.getSubTree(1) );

         if ( leftPrecedence > operatorPrecedence )
         {
            result += "(" + ast2infix( tree.getSubTree(1), indentation, true ) + ")";
         }
         else
         {
            result += ast2infix( tree.getSubTree(1), indentation, true );
         }

         for (int i = 2; i < tree.degree(); i++)
         {
            result += " " + op + " ";

            int rightPrecedence = precedence( tree.getSubTree(i) );

            if ( rightPrecedence >= operatorPrecedence )
            {
               result += "(" + ast2infix( tree.getSubTree(i), indentation, true ) + ")";
            }
            else
            {
               result += ast2infix( tree.getSubTree(i), indentation, true );
            }
         }
      }

      return result;
   }//builtin()


   // http://www.java-tips.org/java-se-tips/java.lang/what-is-java-operator-precedence.html
   // http://www.cs.princeton.edu/introcs/11precedence/
   private static int precedence(Tree tree)
   {
      int result = 0;  // "highest" precedence (i.e., constants)

      String op = null;

      // get the operator from the tree
      if ( tree.getElement().equals("apply") )
      {
         op = tree.getSubTree(0).getElement();  // built-in operator
      }
      else
      {
         op = tree.getElement(); // special form operator
      }

      if ( op.equals("!") )
      {
         result = 2;
      }
      else if ( op.equals("^") )
      {
         result = 3;
      }
      else if ( op.equals("*")
             || op.equals("/")
             || op.equals("%") )
      {
         result = 4;
      }
      else if ( op.equals("+")
             || op.equals("-") )
      {
         result = 5;
      }
      else if ( op.equals("<")
             || op.equals(">")
             || op.equals("<=")
             || op.equals(">=") )
      {
         result = 7;
      }
      else if ( op.equals("==")
             || op.equals("!=") )
      {
         result = 8;
      }
      else if ( op.equals("&&") )
      {
         result = 11;
      }
      else if ( op.equals("||") )
      {
         result = 12;
      }
      else if ( op.equals("var") || op.equals("set") )
      {
         result = 14;
      }

      return result;
   }

}//AST2infix