/**
   This ast2infix() method takes a BTree for the language
   of arithmetic expressions and converts it into a string
   that contains the infix version of the expression.

   This version of ast2infix() uses knowledge about operator precedence
   (in infix notation) to use the minimun number of parentheses.
*/

public class AST2infix2
{
   /**
      This ast2infix() method is essentially a in-order
      traversal of the binary tree.
   */
   public static String ast2infix(BTree tree)
   {
      String result = "";

      if ( tree.degree() == 0 )  // constant term (stops the recursion)
      {
         result += tree.getElement();
      }
      else // binary operator
      {
         int middlePrecedence = precedence( tree.getElement() );
         int leftPrecedence = precedence( tree.getLeftTree().getElement() );

         if ( leftPrecedence > middlePrecedence )
         {
            result += "(" + ast2infix( tree.getLeftTree() ) + ")";
         }
         else
         {
            result += ast2infix( tree.getLeftTree() );
         }

         result += " " + tree.getElement() + " ";

         int rightPrecedence = precedence( tree.getRightTree().getElement() );

         if ( rightPrecedence >= middlePrecedence )
         {
            result += "(" + ast2infix( tree.getRightTree() ) + ")";
         }
         else
         {
            result += ast2infix( tree.getRightTree() );
         }
      }
      return result;
   }//ast2infix()


   // http://www.java-tips.org/java-se-tips/java.lang/what-is-java-operator-precedence.html
   // http://www.cs.princeton.edu/introcs/11precedence/
   private static int precedence(String op)
   {
      int result = 0;  // "highest" precedence (i.e., constants)

      if ( op.equals("*")
        || op.equals("/")
        || op.equals("%") )
      {
         result = 4;
      }
      else if ( op.equals("+")
             || op.equals("-") )
      {
         result = 5;
      {

      return result;
   }

}//AST2infix2