/*
   A Tree is defined by the following grammar.

     Tree ::= '(' String Tree+ ')'
            | String

   where '(' and ')' are literals, and Tree+ means "one or more".

   This language also assumes that all tokens have white space around them.

   The static method getTree() is essentially a recursive descent
   parser for this Tree langauge.
*/
#include "Tokenizer.h"
#include "Tree.h"


Tree* getTree(Tokenizer* tokens)
{
   Tree* result;
   std::string token = tokens->nextToken(); // consume one token
   if ( 0==token.compare("(") )             // look for a parenthesized tree
   {
      //std::cout << "creating internal node with " << tokens->peekToken() << "\n";
      result = new Tree( tokens->nextToken() ); // consume the root of the tree
      (*result).addSubTree( getTree(tokens) );  // consume first sub tree
      token = tokens->peekToken();              // one character look ahead
      while ( ! (0==token.compare(")")) )
      {
         (*result).addSubTree( getTree(tokens) ); // consume the sub tree
         token = tokens->peekToken();             // one character look ahead
      }
      tokens->match(")");            // consume the matching ")"
   }
   else
   {
      //std::cout << "creating     leaf node with " << token << "\n";
      result = new Tree(token);       // the tree must be just the root
   }
   return result;
}//getTree()



Tree* buildTree(std::string expression)
{
   Tokenizer* tokens = new Tokenizer(expression);
   //std::cout << *tokens << "\n";

   Tree* result = getTree(tokens); // parse the tokens

   if ( tokens->hasToken() )  // there shouldn't be any more tokens
   {
      throw "syntax error: unexpected input: " + tokens->peekToken() + "\n";
   }

   return result;
}//buildTree()
