/**
   This program evaluates (interprets) a Tree
   that is defined by the following grammar.

        Exp ::= '(' BinOp Exp Exp ')'
              | INTEGER

     BinOp  ::= '+' | '-' | '*' | '/' | ' %'

    INTEGER ::= [0-9]+
*/
#include <cstdlib>

/*
   The evaluate() function does a post-order traversal of a tree
   that represents an expression (i.e., an "expression tree") and
   calculates the value of the expression represented by the tree.
*/
int evaluate(Tree* tree)
{
   int result = 0;

   if ( tree->degree() == 0 )  // constant term (stops the recursion)
   {
      result = atoi( (tree->getElement()).c_str() );
   }
   else
   {
      std::string op = tree->getElement();

      int resultL = evaluate( tree->getSubTree(0) );
      int resultR = evaluate( tree->getSubTree(1) );

      if ( 0==op.compare("+") )
      {
        result = resultL + resultR;
      }
      else if ( 0==op.compare("-") )
      {
        result = resultL - resultR;
      }
      else if ( 0==op.compare("*") )
      {
        result = resultL * resultR;
      }
      else if ( 0==op.compare("/") )
      {
        result = resultL / resultR;
      }
      else if ( 0==op.compare("%") )
      {
        result = resultL % resultR;
      }
   }

   return result;
}
