/**
   This program evaluates (interprets) a BTree
   that is defined by the following grammar.

        Exp ::= '(' BinOp Exp Exp ')'
              | INTEGER

     BinOp  ::= '+' | '-' | '*' | '/' | '%'

    INTEGER ::= [0-9]+
*/

public class Evaluate
{

   /**
      The evaluate() method does a post-order traversal of a binary tree
      that represents an expression (i.e., an "expression tree") and
      calculates the value of the expression represented by the tree.
   */
   public static int evaluate(BTree tree)
   {
      int result = 0;

      if ( tree.degree() == 0 )  // constant term (stops the recursion)
      {
         result = Integer.parseInt( tree.getElement() );
      }
      else
      {
         String op = tree.getElement();

         int resultL = evaluate( tree.getLeftTree() );
         int resultR = evaluate( tree.getRightTree() );

         if ( op.equals("+") )
         {
           result = resultL + resultR;
         }
         else if ( op.equals("-") )
         {
           result = resultL - resultR;
         }
         else if ( op.equals("*") )
         {
           result = resultL * resultR;
         }
         else if ( op.equals("/") )
         {
           result = resultL / resultR;
         }
         else if ( op.equals("%") )
         {
           result = resultL % resultR;
         }
      }

      return result;
   }

}//Evaluate