/*
  This program evaluates the following BTree, which represents
  the arithmetic expression 4*5 + 100/(15-5).

                 +
                / \
              /     \
             *       /
            / \    /   \
          4    5  100    -
                        / \
                      /     \
                     15      5
*/

public class Example
{
   public static void main(String[] args)
   {
      String exp = "( + ( * 4 5 ) ( / 100 ( - 15 5 ) ) )";

      BTree btree = BuildBTree.buildBTree( exp );

      int result = Evaluate.evaluate( btree );

      System.out.println( "4*5+100/(15-5) = " + result + "\n" );


      // The "pretty printer" prints out the tree in a nice format.
      // In a sense, it reverses the process of parsing (and building)
      // the tree, so a "pretty-printer" is sometimes called an "unparser".
      System.out.println( PrettyPrinter.prettyPrint( btree ) + "\n" );
      System.out.println( PrettyPrinter2.prettyPrint( btree ) + "\n" );


      // The ast2infix() method prints out, in infix notation, the expression
      // represented by the expression tree . In a sense, the builBTree()
      // and ast2infix() methods combine to define a language "translator".
      // They translates from the prefix expression language to the
      // infix expression language.
      System.out.println( AST2infix.ast2infix( btree ) + "\n" );
      System.out.println( AST2infix2.ast2infix( btree ) );


   }//main

}//Example
