/*
  This program uses a small "Domain Specific Language" (DSL) to
  solve the problem of instantiating binary trees. Binary trees
  are represented by strings in a simple BTree language and a
  "parser" reads these strings and builds the appropriate binary tree.

                 a
                / \
              /     \
             b       c
            / \        \
          /     \        \
         d       e        f
                         /
                       /
                      g
                       \
                         \
                           h
*/

public class Example2
{
   public static void main(String[] args)
   {
      String tree = "( a ( b d e ) ( c () ( f ( g () h ) () ) ) )";

      BTree btree = BuildBTree.buildBTree( tree );

      // Let us traverse the tree so that we can convince
      // ourselves that it was constructed correctly.
      // We should get "d b e a c g h f".
      System.out.println( "  In-order traversal = " + Traverse.inOrder( btree ) + "\n" );
      System.out.println( " Pre-order traversal = " + Traverse.preOrder( btree ) + "\n" );
      System.out.println( "Post-order traversal = " + Traverse.postOrder( btree ) + "\n" );


      // The "pretty printer" prints out the tree in a nice format.
      // In a sense, it reverses the process of parsing (and building)
      // the tree, so a "pretty-printer" is sometimes called an "unparser".
      System.out.println( PrettyPrinter.prettyPrint( btree ) + "\n" );

   }//main

}//Example2
