/*
   This program parses and then evaluates strings from Language-11.
*/
#include "BuildTree.h"
#include "PrettyPrinter3.h"
#include "AST2infix.h"
struct Environment;   // incomplete type definition
#include "IPEP.h"
#include "Value.h"
#include "Environment.h"
#include "Evaluate.h"
//#include "Evaluate-heap.h"

int main()
{
   // Demonstrate a function parameter (using static scope)
   std::string expr1 = "( prog \
                         ( define w 300 ) \
                         ( define f1 ( lambda x ( begin \
                                                ( var f2 ( lambda y ( + ( + x y ) w ) ) )  \
                                                ( apply f3 f2 ) ) ) ) \
                         ( define f3 ( lambda z ( begin \
                                                ( var w 400 ) \
                                                ( apply z 20 ) ) ) ) \
                         ( apply f1 1 ) )";


   // This is realy language-12. It has a function return value.
   std::string expr2 = "( prog \
                         ( define add \
                            ( lambda  x \
                              ( begin \
                                ( var r ( lambda y ( + x y ) ) ) \
                                r ) ) ) \
                         ( begin \
                            ( var add2 ( apply add 2 ) ) \
                            ( apply add2 3 ) ) )";


   std::string expr = expr2;

   Tree* ast;
   try
   {
      // build the abstract syntax tree that represents the expression
      ast = buildTree( expr );
   }
   catch(std::string e)
   {
      std::cout << e << "\n";
      exit(1);
   }

   // print the original expression
   //std::cout << expr << "\n\n";
   // pretty-print the abstract syntax tree
   std::cout << prettyPrint( ast ) << "\n\n";
   // print the infix version of the expression
   std::cout << ast2infix( ast ) << "\n\n";

   try
   {
      // evaluate the expression (interpret the AST)
      Value value = evaluate( ast );

      std::cout << "result = " << value << "\n";
   }
   catch(std::string e)
   {
      std::cout << e << "\n";
      exit(1);
   }

   return 0;
}//main
