/*
   This program parses and then evaluates strings from Language-2-C++.

   This program evaluates the following Tree, which represents
   the arithmetic expression 4*5 + 100/(15-5).

                 +
                / \
              /     \
             *       /
            / \    /   \
          4    5  100    -
                        / \
                      /     \
                     15      5
*/
#include "BuildTree.h"
#include "PrettyPrinter3.h"
#include "AST2infix.h"
#include "Evaluate.h"

int main()
{
   std::string expr1 = "( + ( * 4 5 ) ( / 100 ( - 15 5 ) ) )";

   std::string expr2 = "( * ( * ( * 4 5 ) 6 ) ( + ( % 11 3 ) ( - 15 5 ) ) )";

   std::string expr3 = "( + \
                        ( + \
                          ( + \
                            ( * ( * ( * 40 50 ) 60 ) ( + ( % 110 30 ) ( - 150 50 ) ) ) \
                            ( * ( * ( * 40 50 ) 60 ) ( + ( % 110 30 ) ( - 150 50 ) ) ) \
                          ) \
                          ( + \
                            ( * ( * ( * 40 50 ) 60 ) ( + ( % 110 30 ) ( - 150 50 ) ) ) \
                            ( * ( * ( * 40 50 ) 60 ) ( + ( % 110 30 ) ( - 150 50 ) ) ) \
                          ) \
                        ) \
                        ( + \
                          ( + \
                            ( * ( * ( * 40 50 ) 60 ) ( + ( % 110 30 ) ( - 150 50 ) ) ) \
                            ( * ( * ( * 40 50 ) 60 ) ( + ( % 110 30 ) ( - 150 50 ) ) ) \
                          ) \
                          ( + \
                            ( * ( * ( * 40 50 ) 60 ) ( + ( % 110 30 ) ( - 150 50 ) ) ) \
                            ( * ( * ( * 40 50 ) 60 ) ( + ( % 110 30 ) ( - 150 50 ) ) ) \
                          ) \
                        ) \
                        )";

   std::string expr = expr1;

   Tree* ast;
   try
   {
      // build the abstract syntax tree that represents the expression
      ast = buildTree( expr );
   }
   catch(std::string e)
   {
      std::cout << e << "\n";
      exit(1);
   }

   // print the original expression
   std::cout << expr << "\n\n";
   // pretty-print the abstract syntax tree
   std::cout << prettyPrint( ast ) << "\n\n";
   // print the infix version of the expression
   std::cout << ast2infix( ast ) << "\n\n";

   // evaluate the expression (interpret the AST)
   int value = evaluate( ast );

   std::cout << "result = " << value << "\n";

   return 0;
}//main
