/*
   This program parses, type checks, and then evaluates strings from Language-10.
*/

public class Examples
{
   public static void main(String[] args)
   {
      // Show that this language has static scope (final result should be 100).
      String expr1 = "( prog " +
                       "( define int x ( + 10 100 ) ) " +
                       "( define bool b ( && true ( < x 1000 ) ) ) " +
                       "( define func f1 ( lambda ( signature int int ) y " +
                                           "( + y x ) ) ) " +
                       "( define func f2 ( lambda ( signature bool int ) z " +
                                           "( >= z 0 ) ) ) " +
                       "( || ( apply f2 ( apply f1 1 ) ) ( == false ( apply f2 ( - 0 1 ) ) ) ) )";

      // recursive function
      String expr2 = "( prog " +
                       "( define func even ( lambda ( signature bool int ) n " +
                             "( && ( != n 1 ) " +       // use short-circuiting
                                  "( || ( == n 0 ) " +  // use short-circuiting
                                       "( apply even ( - n 2 ) ) ) ) ) ) " +
                       "( apply even 7 ) )";

      // mutually recursive functions
      String expr3 = "( prog " +
                       "( define func even ( lambda ( signature bool int ) n " +
                             "( || ( == n 0 ) " +       // use short-circuiting
                                  "( apply odd ( - n 1 ) ) ) ) ) " +
                       "( define func odd ( lambda ( signature bool int ) n " +
                            "( && ( >  n 0 ) " +        // use short-circuiting
                                 "( || ( == n 1 ) " +   // use short-circuiting
                                      "( apply even ( - n 1 ) ) ) ) ) ) " +
                       "( apply even 8 ) )";


      String expr = expr1;

      // build the abstract syntax tree that represents the expression
      Tree ast = BuildTree.buildTree( expr );

      // print the original expression
      //System.out.println( expr + "\n" );
      // pretty-print the abstract syntax tree
      System.out.println( PrettyPrinter3.prettyPrint( ast ) + "\n" );
      // print the infix version of the expression
      System.out.println( AST2infix.ast2infix( ast ) + "\n" );

      try
      {
         // run the type checker
         if ( TypeChecker.typeCheck( ast ) )
         {
            TypeRemoval.removeTypes( ast );

            // pretty-print the abstract syntax tree
            System.out.println( PrettyPrinter3.prettyPrint( ast ) + "\n" );

            // evaluate the expression (interpret the AST)
            Value value = Evaluate.evaluate( ast );

            System.out.println( "result = " + value );
         }
         else
         {
            System.out.println("There were type erors.");
         }
      }
      catch (EvalException e)
      {
         e.printStackTrace();
      }
   }//main

}//Examples
