/*
   This program parses and then evaluates strings from Language-9.
*/

public class Examples
{
   public static void main(String[] args)
   {
      String expr1 = "( prog " +
                       "( define f ( lambda x ( begin " +
                                               "( var g ( lambda y ( + 10 y ) ) ) " +
                                               "( apply g x ) ) ) ) " +
                       "( apply f 5 ) )";

      // Show that this language has dynamic scope (final result should be 10).
      String expr2 = "( prog " +
                       "( define f1 ( lambda x 100 ) ) " +
                       "( define f2 ( lambda y ( begin " +
                                              "( var f1 ( lambda w 10 ) ) " +
                                              "( apply f3 0 ) ) ) ) " +
                       "( define f3 ( lambda z ( apply f1 0 ) ) ) " +
                       "( apply f2 0 ) )";


      // Show that this language has dynamic scope.
      // (With static scope, this example would be an infinite loop.)
      String expr3 = "( prog " +
                       "( define f1 ( lambda x ( apply f2 0 ) ) ) " +
                       "( define f2 ( lambda y ( begin " +
                                              "( var f1 ( lambda w 10 ) ) " +
                                              "( apply f3 0 ) ) ) ) " +
                       "( define f3 ( lambda z ( apply f1 0 ) ) ) " +
                       "( apply f1 0 ) )";


      // Another (convoluted) example demonstrating dynamic scope.
      // (Notice that, with static scope, h is undefined in f, and x is undefined in h.)
      String expr4 =
        "( prog " +
          "( define f ( lambda x ( apply h x ) ) ) " +  // non local reference to h
          "( define g ( lambda y ( begin " +
                                 "( var h ( lambda z ( + z x ) ) ) " + // non local ref to x
                                 "( apply f y ) ) ) ) " +
          "( apply g 5 ) )";


      // Show that functions are not first class in this language.
      String expr5 = "( prog " +
                       "( define n 2 ) " +
                       "( define f ( lambda x ( * n x ) ) ) " +
                       "( define g ( lambda h ( apply h 5 ) ) ) " +
                       "( apply g f ) )";  // try to pass f as an argument

      // Show that functions are not first class in this language.
      String expr6 = "( prog " +
                       "( define f ( lambda x ( begin " +
                                               "( var g ( lambda y ( + x y ) ) ) " +
                                               "g ) ) ) " +  // try to return g
                       "( apply f 5 ) )";


      String expr = expr2;

      // build the abstract syntax tree that represents the expression
      Tree ast = BuildTree.buildTree( expr );

      // print the original expression
      System.out.println( expr + "\n" );
      // pretty-print the abstract syntax tree
      System.out.println( PrettyPrinter3.prettyPrint( ast ) + "\n" );
      // print the infix version of the expression
      System.out.println( AST2infix.ast2infix( ast ) + "\n" );

      try
      {
         // evaluate the expression (interpret the AST)
         Value value = Evaluate.evaluate( ast );

         System.out.println( "result = " + value );
      }
      catch (EvalException e)
      {
         e.printStackTrace();
      }
   }//main

}//Examples