/*
   This program parses, then evaluates, a number of examples of
   strings from the language of binary arithmetic and boolean
   expressions.
*/

public class Examples
{
   public static void main(String[] args)
   {
      String[] exprs = {
                         "( + 2 ( * 5 5 ) )",
                         "( + ( * 5 5 ) 2 )",
                         "( - ( * ( + 3 5 ) 4 ) 9 )",
                         "( - 9 ( * 4 ( + 5 3 ) ) )",
                         "( + ( / ( * 32 41 ) 2 ) ( * 12 48 ) )",
                         "( ! false () )",
                         "( || true false )",
                         "( ! ( && true false ) () )",
                         "( < 3 5 )",
                         "( && ( < 3 5 ) ( ! ( > 3 0 ) () ) )",
                         "( == ( + 4 2 ) ( * 2 3 ) )",
                         "( == ( + 4 2 ) ( < 2 3 ) )",
                         "( == ( > 4 2 ) ( < 2 3 ) )"
                       };

      // iterate through the array of expressions
      for (int i = 0; i < exprs.length; i++)
      {
         // build the abstract syntax tree that represents the expression
         BTree ast = BuildBTree.buildBTree( exprs[i] );

         // print the original expression
         System.out.println( exprs[i] + "\n" );
         // pretty-print the abstract syntax tree (expression tree)
         // three different ways
         System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );
         System.out.println( PrettyPrinter2.prettyPrint( ast ) + "\n" );
         System.out.println( PrettyPrinter3.prettyPrint( ast ) + "\n" );
         // print the infix version of the expression
         System.out.println( AST2infix.ast2infix( ast ) + "\n" );

         // evaluate the expression (interpret the AST)
         Value value = Evaluate.evaluate( ast );

         System.out.println( value );
         /*
         if ( value.tag.equals("bool") )
            System.out.println( value.valueB );
         else
            System.out.println( value.valueI );
         */
         System.out.println("===================================");
      }
   }//main

}//Examples
