/*
   Antlr grammar errors can show up when,

   1. compiling the grammar (with antlr.jar),
   2. compiling the lexer and parser (with javac.exe),
   3. compiling a client program (with javac.exe),
   4. running the client program (with java.exe).

   There are four errors in the following grammar and its actions.
   Fix the errors, one at a time in the proper order, and see that
   with each fix, one step of (the four step) client build process
   completes without error.
*/
grammar Language1;

stree [int result]     /* 3. Missing "returns" keyword */
   : tr EOF {$result=$tr.result;}
     /* 1. Missing (Antlr) semi-colon. */

tr returns [int result]
   : '(' ID a=tr {$result=$a.result;} (b=tr {$result+=$b.result;})* ')'
   | ID {System.out.println("leaf node = " + $ID); /* 4. Missing text field access in ID */
         $result=1;}
   ;

ID : ('a'..'z'|'A'..'Z')+
   ;

WS : (' '|'\t'|'\n'|'\r')+ {skip() } /* 2. Missing (Java) semi-colon */
   ;
