/**
   The prettyPrint() method takes a BTree and
   converts it into a well formated string.

   This pretty-printer puts the left node "inline" with the
   root and it also handles the special case when depth=1.
*/

public class PrettyPrinter2
{
   public static String prettyPrint(BTree tree)
   {
      return prettyPrint(tree, 0, true);
   }

   /**
      This prettyPrint() method is essentially a pre-order
      traversal of the tree.
   */
   private static String prettyPrint(BTree tree, int indentation, boolean inline)
   {
      String indent = "";
      for (int i = 0; i < indentation; i++)
         indent += " ";

      String result = "";

      if (tree.degree() == 0)  // constant term (stops the recursion)
      {
         result += tree.getElement();
      }
      else if ( tree.depth() == 1 )
      {
         result += "(" + tree.getElement();

         if ( tree.getLeftTree() != null )
            result += " " + tree.getLeftTree().getElement();
         else
            result += " ()";
         if ( tree.getRightTree() != null )
            result += " " + tree.getRightTree().getElement();
         else
            result += " ()";

         result += ")";
      }
      else
      {
         result += "(" + tree.getElement() + " ";

         int indentation2 = indentation + 2 + tree.getElement().length();

         if ( tree.getLeftTree() != null )
            result += prettyPrint( tree.getLeftTree(), indentation2, true ) + "\n";
         else
            result += "()\n";
         if ( tree.getRightTree() != null )
            result += prettyPrint( tree.getRightTree(), indentation2, false ) + "\n";
         else
         {
            String indent2 = "";
            for (int i = 0; i < indentation+2+tree.getElement().length(); i++)
               indent2 += " ";
            result += indent2 + "()\n";
         }

         result += indent + ")";
      }

      if ( ! inline )
         result = indent + result;

      return result;
   }

}//PrettyPrinter2