/*
  This program interprets Language-4-infix by using Antlr
  to parse infix expressions into an abstract syntax tree,
  and then using our original interpreter for Lnguage-4 to
  interpret the AST.

  But rather than using Antlr's AST directly, we use Antlr's
  text description of the AST (after a bit a processing) and
  build our AST using BuildBTree.java. (as with Language-4).

  A good next step would be to learn the structure of Antlr's
  AST and modify our tree walking interpreters (Evaluate.java,
  AST2infix.java, and PrettyPrinter.java) to walk Antlr's AST
  directly. (Or, better yet, write an Antlr tree grammar that
  walks Antlr's AST and implements the interpreter.)
*/
import org.antlr.runtime.*;

public class TestParser_AST_eval
{
   public static void main(String[] args)
   {
      String[] exprs =
      { // The same examples we used with Language-4.
        "{{x=2; y=2}; z= x+y}",
        "{x=4; {y=false; y||2+x==2*3}}",
        "{x={z=2; z+z}; {y=false; {u=2; {v=y||u+4==u*3; v}}}}",
        "x={y=2; z=y+3}",
        // illegal expressions
        //"z = (x = y = 2) + (y + 3 * x)",
        "{x = false; 1 + x}",
        "(2 * 2 != 5) + 1 / 0"
      };

      for (int i = 0; i < exprs.length; i++)
      {
         Language4_infix_ASTLexer lexer = new Language4_infix_ASTLexer(new ANTLRStringStream(exprs[i]));
         Language4_infix_ASTParser parser = new Language4_infix_ASTParser(new CommonTokenStream(lexer));

         System.out.println( "Parsing: " + exprs[i] );
         try
         {
            // The result from this parse is a String.
            Language4_infix_ASTParser.expression_return r = parser.expression();
            String s = r.result;
            System.out.println( s );

            // prep the string for our stupid tokenizer
            String newS = "";
            for (int j = 0; j < s.length(); j++)
            {
               char c = s.charAt(j);
               if (c == '(')
               {
                  newS += "( "; // put spaces after ('s
               }
               else if (c == ')')
               {
                  newS += " )"; // put spaces before )'s
               }
               else
               {
                  newS += c;
               }
            }
            newS = newS.toLowerCase();
            System.out.println( newS );

            // build our abstract syntax tree
            BTree ast = BuildBTree.buildBTree( newS );

            // pretty-print the abstract syntax tree
            System.out.println( PrettyPrinter3.prettyPrint( ast ) + "\n" );
            // print the infix version of the expression
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // evaluate the expression (interpret the AST)
            Value value = Evaluate.evaluate( ast );
            // print out the result
            System.out.println( "result = " + value );
         }
         catch (Exception e)
         {
            System.out.println(e);
         }
         System.out.println();
      }
   }//main

}//TestParser_AST_eval
