/**
   This very basic tokenizer only does part of the job of a real
   tokenizer. It takes an input string of characters and converts
   it to an array of tokens, but it does not classify the tokens
   into token types. And this tokenizer does not use a grammar to
   define tokens, it just uses white space to delimit the "tokens".
*/
#include <string>
#include <vector>
#include <sstream>
#include <iostream>

class Tokenizer
{
  private:
    std::vector<std::string> tokens;
    unsigned int currentToken;

  public:

   /*
      This constructor converts a string into an array of tokens.
      Whitespace characters are used as token delimiters.

      Got this from
        http://oopweb.com/CPP/Documents/CPPHOWTO/Volume/C++Programming-HOWTO-7.html
   */
   Tokenizer(std::string str) :
   currentToken(0)
   {
      std::string temp;
      std::stringstream ss(str);  // insert the string into a stream

      while (ss >> temp)
         tokens.push_back(temp);
   }

   /*
      This method "consumes" the current token.
   */
   std::string nextToken()
   {
      return tokens[currentToken++];
   }

   /*
      This method both tests and consumes the current token.
   */
   bool match(std::string tk)
   {
      return 0 == tk.compare( tokens[currentToken++] );
   }

   /*
      This method allows you to look at the current
      token without consuming it.
   */
   std::string peekToken()
   {
      return tokens[currentToken];
   }

   /*
      This method allows you to look at the token after
      the current token (without consuming any token).
   */
   std::string peek2Token()
   {
      return tokens[currentToken+1];
   }

   /*
      Returns false if all of the tokens have been "consumed".
   */
   bool hasToken()
   {
      return ( currentToken < tokens.size() );
   }

   /*
      Use this operator for information purposes. It
      allows you to see how a string gets "tokenized".
   */
   friend std::ostream& operator<<(std::ostream& os, const Tokenizer& tkr)
   {
      os << "index = " << tkr.currentToken << ", tokens = [";
      os << tkr.tokens[0];
      for(unsigned int i = 1; i < tkr.tokens.size(); i++)
         os << ", " << tkr.tokens[i];
      os << "]";

      return os;
   }
};


