/**
   This class defines the nodes for a tree data structure.

   A "label" for each node is stored as a String.

*/
#include <string>
#include <vector>

class Tree
{
 private:
   std::string element;
   std::vector<Tree*> subTrees;

 public:
   Tree(std::string e) :
   element(e)
   { }

   std::string getElement()
   {
      return element;
   }

   void addSubTree(Tree* tree)
   {
      subTrees.push_back( tree );
   }

   Tree* getSubTree(int i)
   {
      return subTrees[i];
   }

   /*
      Compute the number of subtrees below this node.
   */
   int degree()
   {
      return subTrees.size();
   }

   /*
      Calculate the depth of the subtree below this node.
   */
   int depth()
   {
      int result = 0;
      if (degree() > 0)
      {
         int max = 0;
         for (int i = 0; i < degree(); i++)
         {
            int temp = (*getSubTree(i)).depth();
            if (temp > max) max = temp;
         }
         result = 1+max;
      }
      return result;
   }
};
