/*
   This program removes type information from an AST for Language-10t,
   thereby creating an instance of an AST for Language-10.

       Prog ::= '(' 'prog' Define* Exp ')'

     Define ::= '(' 'define' Type VARIABLE NExp ')'       // variables have a "type"
              | '(' 'define' 'func' FUNC-NAME Lambda ')'  // functions have a "signature"

     Lambda ::= '(' 'lambda' Signature VARIABLE Exp ')'

  Signature ::= '(' 'signature' Type Type ')'  // return type followed by parameter type

       Type ::= 'bool' | 'int'

        Exp ::= Begin | Var | NExp

      Begin ::= '(' 'begin' Exp  Exp+ ')'

        Var ::= '(' 'var' Type VARIABLE NExp ')'       // variables have a "type"
              | '(' 'var' 'func' FUNC-NAME Lambda ')'  // functions have a "signature"

       NExp ::= Apply | Set | Print | AExp | BExp
              | VARIABLE | INTEGER | BOOLEAN

      Apply ::= '(' 'apply' FUNC-NAME NExp ')'

        Set ::= '(' 'set' VARIABLE NExp ')'

      Print ::= '(' 'print' NExp ')'

       AExp ::= '(' BinOp NExp NExp ')'

     BinOp  ::= '+' | '-' | '*' | '/' | '%' | '^'

       BExp ::= '(' BoolOp NExp NExp+ ')'
              | '(' '!' NExp ')'
              | '(' RelOp NExp NExp ')'
              | '('  EqOp NExp NExp ')'

     BoolOp ::= '&&' | '||'
     RelOp  ::= '<' | '>' | '<=' | '>='
      EqOp  ::= '==' | '!='

    INTEGER ::= [0-9]+
    BOOLEAN ::= 'true' | 'false'
   VARIABLE ::= [a-zA-Z][a-zA-Z0-9]*
  FUNC-NAME ::= [a-zA-Z][a-zA-Z0-9]*

*/

public class TypeRemoval
{
   public static void removeTypes(Tree tree)
   {
      removeProg( tree );
   }//removeTypes()


   private static void removeProg(Tree tree)
   {
      for (int i = 0; i < tree.degree()-1; i++)
      {
         removeDefine( tree.getSubTree(i) );
      }

      removeE( tree.getSubTree(tree.degree()-1) );

   }//removeProg()


   private static void removeDefine(Tree tree)
   {
      // determine if the definition is a function
      if ( tree.getSubTree(0).getElement().equals("func") )
      {
         // remove the Signature sub tree from the lambda
         tree.getSubTree(2).removeSubTree(0);
         // remove the type from define
         tree.removeSubTree(0);
      }
      else // not a function
      {
         // remove any type information in the expression
         removeN( tree.getSubTree(2) );
         // remove the type from define
         tree.removeSubTree(0);
      }
   }//removeDefine()


   private static void removeE(Tree tree)
   {
      String node = tree.getElement();

      if ( node.equals("begin") )
      {
         removeBegin(tree);
      }
      else if ( node.equals("var") )
      {
         removeVar(tree);
      }
      else
      {
         removeN(tree);
      }
   }//removeE()


   private static void removeBegin(Tree tree)
   {
      // evaluate each expression
      for (int i = 0; i < tree.degree(); i++)
      {
         removeE( tree.getSubTree(i) );
      }
   }//removeBegin()


   private static void removeVar(Tree tree)
   {
      // determine if the definition is a function
      if (tree.getSubTree(0).getElement().equals("func"))
      {
         // remove the Signature sub tree from the lambda
         tree.getSubTree(2).removeSubTree(0);
         // remove the type from define
         tree.removeSubTree(0);
      }
      else // not a function
      {
         // remove any type information in the expression
         removeN( tree.getSubTree(2) );
         // remove the type from define
         tree.removeSubTree(0);
      }
   }//removeVar()


   private static void removeN(Tree tree)
   {
      String node = tree.getElement();

      if ( tree.degree() == 0 )
      {
         // nothing to do
      }
      else if ( node.equals("apply") )
      {
         removeApply(tree);
      }
      else if ( node.equals("set") )
      {
         removeSet(tree);
      }
      else if ( node.equals("print") )
      {
         removePrint(tree);
      }
      else if ( node.equals("+") ||     // arithmetic operators
                node.equals("-") ||
                node.equals("*") ||
                node.equals("/") ||
                node.equals("%") ||
                node.equals("^") )
      {
         removeA(tree);
      }
      else if ( node.equals("&&") ||    // boolean operators
                node.equals("||") ||
                node.equals("!") )
      {
         removeB(tree);
      }
      else if ( node.equals("==") ||    // equality operators
                node.equals("!=") )
      {
         removeEq(tree);
      }
      else if ( node.equals("<") ||     // relational operators
                node.equals(">") ||
                node.equals("<=") ||
                node.equals(">=") )
      {
         removeR(tree);
      }
   }//removeN()


   private static void removeApply(Tree tree)
   {
      removeN( tree.getSubTree(1) );
   }//removeApply()


   private static void removeSet(Tree tree)
   {
      removeN( tree.getSubTree(1) );
   }//removeSet()


   private static void removePrint(Tree tree)
   {
      removeN( tree.getSubTree(0) );
   }//removePrint()


   // Evaluate an arithmetic expression
   private static void removeA(Tree tree)
   {
      removeN( tree.getSubTree(0) );
      removeN( tree.getSubTree(1) );
   }//removeA()


   // Evaluate a boolean expression
   private static void removeB(Tree tree )
   {
      removeN( tree.getSubTree(0) );
      if ( tree.degree() == 2 )  // check for second operand
      {
         removeN( tree.getSubTree(1) );
      }
   }//removeB()


   // Evaluate an equality expression (which is a kind of boolean expression)
   private static void removeEq(Tree tree )
   {
      removeE( tree.getSubTree(0) );
      removeE( tree.getSubTree(1) );
   }//removeEq()


   // Evaluate a relational expression (which is a kind of boolean expression)
   private static void removeR(Tree tree )
   {
      removeN( tree.getSubTree(0) );
      removeN( tree.getSubTree(1) );
   }//removeR()

}//TypeRemoval