/*
   This program removes type information from an AST for Language-14t,
   thereby creating an instance of an AST for Language-14.

       Prog ::= '(' 'prog' Body ')'

       Body ::= Exp+

        Exp ::= Begin | Var | NExp

      Begin ::= '(' 'begin' Body ')'

        Var ::= '(' 'var' Type VARIABLE NExp ')'     // variables have a "type"

       NExp ::= Set | If | While | And | Or | Print
              | Apply | VARIABLE
              | Lambda | INTEGER | BOOLEAN

        Set ::= '(' 'set' VARIABLE NExp ')'

         If ::= '(' 'if' NExp Exp Exp ')'

      While ::= '(' 'while' NExp Exp ')'

        And ::= '(' '&&' NExp NExp NExp* ')'

         Or ::= '(' '||' NExp NExp NExp* ')'

         Eq ::= '(' '==' NExp NExp ')'

      NotEq ::= '(' '!=' NExp NExp ')'

      Print ::= '(' 'print' NExp ')'

      Apply ::= '(' 'apply' NExp NExp* ')'

     Lambda ::= '(' 'lambda' Signature VARIABLE* Exp ')'  // functions have a "signature"

  Signature ::= '(' 'signature' Type Type* ')'   // return type and parameter types

       Type ::= 'bool' | 'int' | Signature

   VARIABLE ::= [a-zA-Z][a-zA-Z0-9]*
    INTEGER ::= [0-9]+
    BOOLEAN ::= 'true' | 'false'
*/

public class TypeRemoval
{
   public static void removeTypes(Tree tree)
   {
      removeProg( tree );
   }//removeTypes()


   private static void removeProg(Tree tree)
   {
      for (int i = 0; i < tree.degree(); i++)
      {
         removeE( tree.getSubTree(i) );
      }
   }//removeProg()


   private static void removeE(Tree tree)
   {
      String node = tree.getElement();

      if ( node.equals("begin") )
      {
         removeBegin(tree);
      }
      else if ( node.equals("var") )
      {
         removeVar(tree);
      }
      else
      {
         removeN(tree);
      }
   }//removeE()


   private static void removeBegin(Tree tree)
   {
      // evaluate each expression
      for (int i = 0; i < tree.degree(); i++)
      {
         removeE( tree.getSubTree(i) );
      }
   }//removeBegin()


   private static void removeVar(Tree tree)
   {
      // remove any type information in the expression
      removeN( tree.getSubTree(2) );
      // remove the type from the Var term
      tree.removeSubTree(0);
   }//removeVar()


   private static void removeN(Tree tree)
   {
      String node = tree.getElement();

      if ( tree.degree() == 0 )
      {
         // nothing to do
      }
      else if ( node.equals("set") )
      {
         removeSet(tree);
      }
      else if ( node.equals("if") )
      {
         removeIf(tree);
      }
      else if ( node.equals("while") )
      {
         removeWhile(tree);
      }
      else if ( node.equals("&&") )
      {
         removeAnd(tree);
      }
      else if ( node.equals("||") )
      {
         removeOr(tree);
      }
      else if ( node.equals("==") )
      {
         removeEq(tree);
      }
      else if ( node.equals("!=") )
      {
         removeNotEq(tree);
      }
      else if ( node.equals("print") )
      {
         removePrint(tree);
      }
      else if ( node.equals("apply") )
      {
         removeApply(tree);
      }
      else if ( node.equals("lambda") )
      {
         removeLambda(tree);
      }
   }//removeN()


   private static void removeSet(Tree tree)
   {
      removeN( tree.getSubTree(1) );
   }//removeSet()


   private static void removeIf(Tree tree )
   {
      removeN( tree.getSubTree(0) );
      removeN( tree.getSubTree(1) );
      removeN( tree.getSubTree(2) );
   }//removeIf()


   private static void removeWhile(Tree tree )
   {
      removeN( tree.getSubTree(0) );
      removeN( tree.getSubTree(1) );
   }//removeWhile()


   private static void removeAnd(Tree tree )
   {
      removeN( tree.getSubTree(0) );
      removeN( tree.getSubTree(1) );
   }//removeAnd()


   private static void removeOr(Tree tree )
   {
      removeN( tree.getSubTree(0) );
      removeN( tree.getSubTree(1) );
   }//removeOr()


   private static void removeEq(Tree tree )
   {
      removeN( tree.getSubTree(0) );
      removeN( tree.getSubTree(1) );
   }//removeEq()


   private static void removeNotEq(Tree tree )
   {
      removeN( tree.getSubTree(0) );
      removeN( tree.getSubTree(1) );
   }//removeNotEq()


   private static void removePrint(Tree tree )
   {
      removeE( tree.getSubTree(0) );
   }//removePrint()


   private static void removeApply(Tree tree)
   {
      if ( tree.getSubTree(0).getElement().equals("lambda") )  // anonymous lambda application
      {
         removeLambda( tree.getSubTree(0) );
      }

      for (int i = 1; i < tree.degree(); i++)
      {
         removeN( tree.getSubTree(i) );
      }
   }//removeApply()


   private static void removeLambda(Tree tree )
   {
      // remove any type information in the expression
      removeN( tree.getSubTree(2) );
      // remove the type from the Lambda term
      tree.removeSubTree(0);
   }//removeLambda()

}//TypeRemoval