/**
   In order to create a single namespace, so that we can have
   first class (global) functions, for this language we extend the
   definition of a value to include a third type, with the tag "lambda".

   The "tree" that we store in a Value with tag "lambda" is a
   reference to the abstract syntax tree of the Lambda term
   that defines the function.
*/

public class Value
{
   public String tag;    // either "int" or "bool" or "lambda"
   public int valueI=0;
   public boolean valueB=false;
   public Tree valueL=null;

   /**
      Construct a Value object that holds an int value.
   */
   public Value(String tag, int value)
   {
      if (tag.equals("int"))
      {
         this.tag = tag;
         valueI = value;
      }
      else
         throw new Error("invalid value tag: " + tag);
   }

   /**
      Construct a Value object that holds a boolean value.
   */
   public Value(String tag, boolean value)
   {
      if (tag.equals("bool"))
      {
         this.tag = tag;
         valueB = value;
      }
      else
         throw new Error("invalid value tag: " + tag);
   }

   /**
      Construct a Value object that holds a lambda value.
   */
   public Value(String tag, Tree value)
   {
      if (tag.equals("lambda"))
      {
         this.tag = tag;
         valueL = value;
      }
      else
         throw new Error("invalid value tag: " + tag);
   }

   public String toString()
   {
      return "[tag->" + tag + ", valueI->" + valueI + ", valueB->" + valueB + ", valueL->" + valueL + "]";
   }

/*
   public String toString()
   {
      String result = "";

      if ( tag.equals("bool") )
      {
         result += valueB;
      }
      else if ( tag.equals("int") )
      {
         result += valueI;
      }
      else if ( tag.equals("lambda") )
      {
         result += valueL;
      }
      else  // bad tag (shouldn't get here)
      {
         result += "[tag->" + tag + ", valueI->" + valueI + ", valueB->" + valueB + ", valueL->" + valueL + "]";
      }

      return result;
   }
*/
}//Value