/**
   This program evaluates (interprets) an AST
   that is defined by the following grammar.

        Ast ::= '(' BinOp Ast Ast ')'
              | '(' '-' Ast ')'
              | INTEGER

     BinOp  ::= '+' | '-' | '*' | '/' | ' %' | '^'

    INTEGER ::= [0-9]+
*/

public class EvaluateAST
{

   /**
      The evaluate() method does a post-order traversal of an AST
      that represents an expression (i.e., an "expression tree") and
      calculates the value of the expression represented by the tree.
   */
   public static int evaluate(Tree tree)
   {
      int result = 0;

      if ( tree.degree() == 0 )  // constant term (stops the recursion)
      {
         result = Integer.parseInt( tree.getElement() );
      }
      else if ( tree.degree() == 1 )  // unary negation
      {
         result = - evaluate( tree.getSubTree(0) );
      }
      else
      {
         String op = tree.getElement();

         int resultL = evaluate( tree.getSubTree(0) );
         int resultR = evaluate( tree.getSubTree(1) );

         if ( op.equals("+") )
         {
            result = resultL + resultR;
         }
         else if ( op.equals("-") )
         {
            result = resultL - resultR;
         }
         else if ( op.equals("*") )
         {
            result = resultL * resultR;
         }
         else if ( op.equals("/") )
         {
            result = resultL / resultR;
         }
         else if ( op.equals("%") )
         {
            result = resultL % resultR;
         }
         else if ( op.equals("^") )
         {
            result = (int)Math.pow(resultL, resultR);
         }
      }

      return result;
   }

}//EvaluateAST