/*
   The static method parse() is a recursive descent parser
   for the grammar in Language-2-infix.txt.

   This parser is only a recognizer for the grammar. This parser
   perfoms no actions. If an expression parses correctly, nothing happens.
   If an expression does not parse, a ParseException gets thrown.

   Use the program TestParser.java to test this parser.
*/

public class Parser
{
   public static void parse(String expression) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(expression);  // Tokenizer is defined
                                                     // in Tokenizer.java
      parseExp(tokens);

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new ParseException("unexpected input: "+tokens.peekToken()+"\n"+tokens);

   }//buildASTree()


   private static void parseExp(Tokenizer tokens) throws ParseException
   {
      parseTerm(tokens);
      while ( tokens.hasToken()
            && (tokens.peekToken().equals("+")
             || tokens.peekToken().equals("-")) )
      {
         tokens.nextToken();
         parseTerm(tokens);
      }
   }//parseExp()


   private static void parseTerm(Tokenizer tokens) throws ParseException
   {
      parseSigned(tokens);
      while ( tokens.hasToken()
            && (tokens.peekToken().equals("*")
             || tokens.peekToken().equals("/")
             || tokens.peekToken().equals("%")) )
      {
         tokens.nextToken();
         parseSigned(tokens);
      }
   }//parseTerm()


   private static void parseSigned(Tokenizer tokens) throws ParseException
   {
      if ( tokens.peekToken().equals("-") )
      {
         tokens.match("-");
      }
      parseFactor(tokens);
   }//parseSigned()


   private static void parseFactor(Tokenizer tokens) throws ParseException
   {
      parseBase(tokens);
      if ( tokens.hasToken() && tokens.peekToken().equals("^") )
      {
         tokens.match("^");
         parseSigned(tokens);
      }
   }//parseFactor()


   private static void parseBase(Tokenizer tokens) throws ParseException
   {
      if ( tokens.peekToken().equals("(") )
      {
         tokens.match("(");
         parseExp(tokens);
         tokens.match(")");
      }
      else if ( tokens.peekToken().matches("^[0-9][0-9]*") )
      {
         tokens.nextToken();
      }
      else
      {
         throw new ParseException("incorrect base: " + tokens.nextToken());
      }
   }//parseBase()

}//Parser