/*
   The static method parse() is a recursive descent parser
   for the grammar in Language-2-infix.txt.

   This parser builds a parse tree. If an expression parses
   correctly, a reference to the parse tree is returned.
   If an expression does not parse, a ParseException gets thrown.

   Use the program TestParser.java to test this parser.
*/

public class Parser2ParseTree
{
   public static Tree parse(String expression) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(expression);  // Tokenizer is defined
                                                     // in Tokenizer.java
      Tree parseTree = parseExp(tokens);

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new ParseException("unexpected input: "+tokens.peekToken()+"\n"+tokens);

      return parseTree;
   }//buildASTree()


   private static Tree parseExp(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree("Exp");
      result.addSubTree( parseTerm(tokens) );
      while ( tokens.hasToken()
            && (tokens.peekToken().equals("+")
             || tokens.peekToken().equals("-")) )
      {
         Tree temp = new Tree( "Exp" );
         temp.addSubTree( result );
         temp.addSubTree( new Tree( "'"+tokens.nextToken()+"'" ) );
         temp.addSubTree( parseTerm(tokens) );
         result = temp;
      }
      return result;
   }//parseExp()


   private static Tree parseTerm(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree("Term");
      result.addSubTree(  parseSigned(tokens) );
      while ( tokens.hasToken()
            && (tokens.peekToken().equals("*")
             || tokens.peekToken().equals("/")
             || tokens.peekToken().equals("%")) )
      {
         Tree temp = new Tree( "Term" );
         temp.addSubTree( result );
         temp.addSubTree( new Tree( "'"+tokens.nextToken()+"'" ) );
         temp.addSubTree( parseSigned(tokens) );
         result = temp;
      }
      return result;
   }//parseTerm()


   private static Tree parseSigned(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree("Signed");
      if ( tokens.peekToken().equals("-") )
      {
         result.addSubTree( new Tree( "'"+tokens.nextToken()+"'" ) );
         result.addSubTree( parseFactor(tokens) );
      }
      else
      {
         result.addSubTree( parseFactor(tokens) );
      }
      return result;
   }//parseSigned()


   private static Tree parseFactor(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree("Factor");
      result.addSubTree( parseBase(tokens) );
      if ( tokens.hasToken() && tokens.peekToken().equals("^") )
      {
         result.addSubTree( new Tree( "'"+tokens.nextToken()+"'" ) );
         result.addSubTree( parseSigned(tokens) );
      }
      return result;
   }//parseFactor()


   private static Tree parseBase(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree("Base");

      if ( tokens.peekToken().equals("(") )
      {
         tokens.match("(");
         result.addSubTree( new Tree("'('") );
         result.addSubTree( parseExp(tokens) );
         tokens.match(")");
         result.addSubTree( new Tree("')'") );
      }
      else if ( tokens.peekToken().matches("^[0-9][0-9]*") )
      {
         Tree temp = new Tree("Integer");
         temp.addSubTree( new Tree( tokens.nextToken() ) );
         result.addSubTree( temp );
      }
      else
      {
         throw new ParseException("incorrect base: " + tokens.nextToken());
      }
      return result;
   }//parseBase()

}//Parser2ParseTree