/*
   The static method parse() is a recursive descent parser
   for the grammar in Language-2-infix.txt.

   This parser returns the value of the expression being parsed.
   In other words, this parser both parses and evaluates at the same time.
   If an expression does not parse, a ParseException gets thrown.

   Use the program TestParser.java to test this parser.
*/

public class Parser2value
{
   public static int parse(String expression) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(expression);  // Tokenizer is defined
                                                     // in Tokenizer.java
      int value = parseExp(tokens);

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new ParseException("unexpected input: "+tokens.peekToken()+"\n"+tokens);

      return value;
   }//buildASTree()


   private static int parseExp(Tokenizer tokens) throws ParseException
   {
      int result = parseTerm(tokens);
      while ( tokens.hasToken()
            && (tokens.peekToken().equals("+")
             || tokens.peekToken().equals("-")) )
      {
         if ( tokens.nextToken().equals("+") )
         {
            result += parseTerm(tokens);
         }
         else
         {
            result -= parseTerm(tokens);
         }
      }
      return result;
   }//parseExp()


   private static int parseTerm(Tokenizer tokens) throws ParseException
   {
      int result = parseSigned(tokens);
      while ( tokens.hasToken()
            && (tokens.peekToken().equals("*")
             || tokens.peekToken().equals("/")
             || tokens.peekToken().equals("%")) )
      {
         String op = tokens.nextToken();
         if ( op.equals("*") )
         {
            result *= parseTerm(tokens);
         }
         else if ( op.equals("/") )
         {
            result /= parseTerm(tokens);
         }
         else if ( op.equals("%") )
         {
            result %= parseTerm(tokens);
         }
      }
      return result;
   }//parseTerm()


   private static int parseSigned(Tokenizer tokens) throws ParseException
   {
      if ( tokens.peekToken().equals("-") )
      {
         tokens.match("-");
         return - parseFactor(tokens);
      }
      else
      {
         return parseFactor(tokens);
      }
   }//parseSigned()


   private static int parseFactor(Tokenizer tokens) throws ParseException
   {
      int result = parseBase(tokens);
      if ( tokens.hasToken() && tokens.peekToken().equals("^") )
      {
         tokens.match("^");
         result = (int)Math.pow( result, parseSigned(tokens) );
      }
      return result;
   }//parseFactor()


   private static int parseBase(Tokenizer tokens) throws ParseException
   {
      int result = 0;

      if ( tokens.peekToken().equals("(") )
      {
         tokens.match("(");
         result = parseExp(tokens);
         tokens.match(")");
      }
      else if ( tokens.peekToken().matches("^[0-9][0-9]*") )
      {
         result = Integer.parseInt( tokens.nextToken() );
      }
      else
      {
         throw new ParseException("incorrect base: " + tokens.nextToken());
      }
      return result;
   }//parseBase()

}//Parser2value